@@ -2,7 +2,7 @@
  *
  *  OBEX Server
  *
- *  Copyright (C) 2007-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2007-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -37,10 +37,184 @@
 
 #include <openobex/obex.h>
 
+#include "bluetooth.h"
 #include "obexd.h"
 #include "obex.h"
+#include "dbus.h"
 #include "logging.h"
 
+static const gchar *opp_record = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>	\
+<record>									\
+  <attribute id=\"0x0001\">							\
+    <sequence>									\
+      <uuid value=\"0x1105\"/>							\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0004\">							\
+    <sequence>									\
+      <sequence>								\
+        <uuid value=\"0x0100\"/>						\
+      </sequence>								\
+      <sequence>								\
+        <uuid value=\"0x0003\"/>						\
+        <uint8 value=\"%u\" name=\"channel\"/>					\
+      </sequence>								\
+      <sequence>								\
+        <uuid value=\"0x0008\"/>						\
+      </sequence>								\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0009\">							\
+    <sequence>									\
+      <sequence>								\
+        <uuid value=\"0x1105\"/>						\
+        <uint16 value=\"0x0100\" name=\"version\"/>				\
+      </sequence>								\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0100\">							\
+    <text value=\"%s\" name=\"name\"/>						\
+  </attribute>									\
+										\
+  <attribute id=\"0x0303\">							\
+    <sequence>									\
+      <uint8 value=\"0x01\"/>							\
+      <uint8 value=\"0x01\"/>							\
+      <uint8 value=\"0x02\"/>							\
+      <uint8 value=\"0x03\"/>							\
+      <uint8 value=\"0x04\"/>							\
+      <uint8 value=\"0x05\"/>							\
+      <uint8 value=\"0x06\"/>							\
+      <uint8 value=\"0xff\"/>							\
+    </sequence>									\
+  </attribute>									\
+</record>";
+
+static const gchar *ftp_record = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>	\
+<record>									\
+  <attribute id=\"0x0001\">							\
+    <sequence>									\
+      <uuid value=\"0x1106\"/>							\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0004\">							\
+    <sequence>									\
+      <sequence>								\
+        <uuid value=\"0x0100\"/>						\
+      </sequence>								\
+      <sequence>								\
+        <uuid value=\"0x0003\"/>						\
+        <uint8 value=\"%u\" name=\"channel\"/>					\
+      </sequence>								\
+      <sequence>								\
+        <uuid value=\"0x0008\"/>						\
+      </sequence>								\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0009\">							\
+    <sequence>									\
+      <sequence>								\
+        <uuid value=\"0x1106\"/>						\
+        <uint16 value=\"0x0100\" name=\"version\"/>				\
+      </sequence>								\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0100\">							\
+    <text value=\"%s\" name=\"name\"/>						\
+  </attribute>									\
+</record>";
+
+static const gchar *pbap_record = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>	\
+<record>									\
+  <attribute id=\"0x0001\">							\
+    <sequence>									\
+      <uuid value=\"0x112f\"/>							\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0004\">							\
+    <sequence>									\
+      <sequence>								\
+        <uuid value=\"0x0100\"/>						\
+      </sequence>								\
+      <sequence>								\
+        <uuid value=\"0x0003\"/>						\
+        <uint8 value=\"%u\" name=\"channel\"/>					\
+      </sequence>								\
+      <sequence>								\
+        <uuid value=\"0x0008\"/>						\
+      </sequence>								\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0009\">							\
+    <sequence>									\
+      <sequence>								\
+        <uuid value=\"0x1130\"/>						\
+        <uint16 value=\"0x0100\" name=\"version\"/>				\
+      </sequence>								\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0100\">							\
+    <text value=\"%s\" name=\"name\"/>						\
+  </attribute>									\
+										\
+  <attribute id=\"0x0314\">							\
+    <uint8 value=\"0x01\"/>							\
+  </attribute>									\
+</record>";
+
+static const gchar *pcsuite_record =
+"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>					\
+<record>									\
+  <attribute id=\"0x0001\">							\
+    <sequence>									\
+      <uuid value=\"00005005-0000-1000-8000-0002ee000001\"/>			\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0004\">							\
+    <sequence>									\
+      <sequence>								\
+        <uuid value=\"0x0100\"/>						\
+      </sequence>								\
+      <sequence>								\
+        <uuid value=\"0x0003\"/>						\
+        <uint8 value=\"%u\" name=\"channel\"/>					\
+      </sequence>								\
+      <sequence>								\
+        <uuid value=\"0x0008\"/>						\
+      </sequence>								\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0005\">							\
+    <sequence>									\
+      <uuid value=\"0x1002\"/>							\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0009\">							\
+    <sequence>									\
+      <sequence>								\
+        <uuid value=\"00005005-0000-1000-8000-0002ee000001\"/> 			\
+        <uint16 value=\"0x0100\" name=\"version\"/>				\
+      </sequence>								\
+    </sequence>									\
+  </attribute>									\
+										\
+  <attribute id=\"0x0100\">							\
+    <text value=\"%s\" name=\"name\"/>						\
+  </attribute>									\
+</record>";
+
 #define TRANSFER_INTERFACE OPENOBEX_SERVICE ".Transfer"
 #define SESSION_INTERFACE OPENOBEX_SERVICE ".Session"
 
@@ -58,9 +232,20 @@ static struct agent *agent = NULL;
 
 struct pending_request {
 	struct server *server;
+	gchar address[18];
 	gint nsk;
 };
 
+struct adapter_any {
+	char *path;		/* Adapter ANY path */
+	GSList *servers;	/* List of servers to register records */
+};
+
+static DBusConnection *connection = NULL;
+static DBusConnection *system_conn = NULL;
+static struct adapter_any *any = NULL;
+static guint listener_id = 0;
+
 static void agent_free(struct agent *agent)
 {
 	g_free(agent->new_folder);
@@ -173,7 +358,7 @@ static void dbus_message_iter_append_dict_entry(DBusMessageIter *dict,
 	dbus_message_iter_close_container(dict, &entry);
 }
 
-static void agent_disconnected(void *user_data)
+static void agent_disconnected(DBusConnection *conn, void *user_data)
 {
 	debug("Agent exited");
 	agent_free(agent);
@@ -318,22 +503,212 @@ static GDBusMethodTable session_methods[] = {
 	{ }
 };
 
-static DBusConnection *connection = NULL;
-
-static DBusConnection *system_conn = NULL;
-
-gboolean manager_init(DBusConnection *conn)
+static gchar *create_xml_record(const char *name,
+			uint16_t service, uint8_t channel)
 {
-	DBG("conn %p", conn);
+	gchar *xml;
 
-	connection = dbus_connection_ref(conn);
-	if (connection == NULL)
+	switch (service) {
+	case OBEX_OPP:
+		xml = g_markup_printf_escaped(opp_record, channel, name);
+		break;
+	case OBEX_FTP:
+		xml = g_markup_printf_escaped(ftp_record, channel, name);
+		break;
+	case OBEX_PBAP:
+		xml = g_markup_printf_escaped(pbap_record, channel, name);
+		break;
+	case OBEX_PCSUITE:
+		xml = g_markup_printf_escaped(pcsuite_record, channel, name);
+		break;
+	default:
+		xml = NULL;
+		break;
+	}
+
+	return xml;
+}
+
+static void add_record_reply(DBusPendingCall *call, gpointer user_data)
+{
+	struct server *server = user_data;
+	DBusMessage *reply = dbus_pending_call_steal_reply(call);
+	DBusError derr;
+	uint32_t handle;
+
+	dbus_error_init(&derr);
+	if (dbus_set_error_from_message(&derr, reply)) {
+		error("Replied with an error: %s, %s",
+				derr.name, derr.message);
+		dbus_error_free(&derr);
+		handle = 0;
+	} else {
+		dbus_message_get_args(reply, NULL,
+				DBUS_TYPE_UINT32, &handle,
+				DBUS_TYPE_INVALID);
+		server->handle = handle;
+
+		debug("Registered: %s, handle: 0x%x, folder: %s",
+				server->name, handle, server->folder);
+	}
+
+	dbus_message_unref(reply);
+}
+
+static gint add_record(const gchar *path,
+		const gchar *xml, struct server *server)
+{
+	DBusMessage *msg;
+	DBusPendingCall *call;
+	gint ret = 0;
+
+	msg = dbus_message_new_method_call("org.bluez", path,
+					"org.bluez.Service", "AddRecord");
+
+	dbus_message_append_args(msg, DBUS_TYPE_STRING, &xml,
+			DBUS_TYPE_INVALID);
+
+	if (dbus_connection_send_with_reply(system_conn,
+				msg, &call, -1) == FALSE) {
+		ret = -1;
+		goto failed;
+	}
+
+	dbus_pending_call_set_notify(call, add_record_reply, server, NULL);
+	dbus_pending_call_unref(call);
+
+failed:
+	dbus_message_unref(msg);
+	return ret;
+}
+
+void register_record(struct server *server, gpointer user_data)
+{
+	gchar *xml;
+	gint ret;
+
+	if (system_conn == NULL)
+		return;
+
+	if (any->path == NULL) {
+		/* Adapter ANY is not available yet: Add record later */
+		any->servers = g_slist_append(any->servers, server);
+		return;
+	}
+
+	xml = create_xml_record(server->name, server->services, server->channel);
+	ret = add_record(any->path, xml, server);
+	g_free(xml);
+}
+
+static void find_adapter_any_reply(DBusPendingCall *call, gpointer user_data)
+{
+	DBusMessage *reply = dbus_pending_call_steal_reply(call);
+	struct server *server;
+	const char *path;
+	gchar *xml;
+	GSList *l;
+	DBusError derr;
+
+	dbus_error_init(&derr);
+	if (dbus_set_error_from_message(&derr, reply)) {
+		error("Replied with an error: %s, %s",
+				derr.name, derr.message);
+		dbus_error_free(&derr);
+		bluetooth_stop();
+		goto done;
+	}
+
+	dbus_message_get_args(reply, NULL,
+			DBUS_TYPE_OBJECT_PATH, &path,
+			DBUS_TYPE_INVALID);
+	any->path = g_strdup(path);
+
+	for (l = any->servers; l; l = l->next) {
+		server = l->data;
+		xml = create_xml_record(server->name,
+				server->services, server->channel);
+		add_record(path, xml, server);
+		g_free(xml);
+	}
+
+done:
+	g_slist_free(any->servers);
+	any->servers = NULL;
+
+	dbus_message_unref(reply);
+}
+static void find_adapter(const char *pattern,
+				DBusPendingCallNotifyFunction function,
+				gpointer user_data)
+{
+	DBusMessage *msg;
+	DBusPendingCall *call;
+
+	debug("FindAdapter(%s)", pattern);
+
+	msg = dbus_message_new_method_call("org.bluez", "/",
+					"org.bluez.Manager", "FindAdapter");
+
+	dbus_message_append_args(msg, DBUS_TYPE_STRING, &pattern,
+			DBUS_TYPE_INVALID);
+
+	dbus_connection_send_with_reply(system_conn, msg, &call, -1);
+	dbus_pending_call_set_notify(call, function, user_data, NULL);
+	dbus_pending_call_unref(call);
+
+	dbus_message_unref(msg);
+}
+
+static gboolean find_adapter_any(gpointer user_data)
+{
+	find_adapter("any", find_adapter_any_reply, user_data);
+
+	return FALSE;
+}
+
+static void name_acquired(DBusConnection *conn, void *user_data)
+{
+	find_adapter_any(NULL);
+	bluetooth_start();
+}
+
+static void name_released(DBusConnection *conn, void *user_data)
+{
+	g_free(any->path);
+	any->path = NULL;
+	bluetooth_stop();
+}
+
+gboolean manager_init(void)
+{
+	DBusError err;
+
+	DBG("");
+
+	any = g_new0(struct adapter_any, 1);
+
+	dbus_error_init(&err);
+
+	connection = g_dbus_setup_bus(DBUS_BUS_SESSION, OPENOBEX_SERVICE, &err);
+	if (connection == NULL) {
+		if (dbus_error_is_set(&err) == TRUE) {
+			fprintf(stderr, "%s\n", err.message);
+			dbus_error_free(&err);
+		} else
+			fprintf(stderr, "Can't register with session bus\n");
 		return FALSE;
+	}
 
 	system_conn = g_dbus_setup_bus(DBUS_BUS_SYSTEM, NULL, NULL);
 	if (system_conn == NULL)
 		return FALSE;
 
+	listener_id = g_dbus_add_service_watch(system_conn, "org.bluez",
+				name_acquired, name_released, NULL, NULL);
+
+	g_idle_add(find_adapter_any, NULL);
+
 	return g_dbus_register_interface(connection, OPENOBEX_MANAGER_PATH,
 					OPENOBEX_MANAGER_INTERFACE,
 					manager_methods, manager_signals, NULL,
@@ -342,7 +717,7 @@ gboolean manager_init(DBusConnection *conn)
 
 void manager_cleanup(void)
 {
-	DBG("conn %p", connection);
+	DBG("");
 
 	g_dbus_unregister_interface(connection, OPENOBEX_MANAGER_PATH,
 						OPENOBEX_MANAGER_INTERFACE);
@@ -352,6 +727,13 @@ void manager_cleanup(void)
 	if (agent)
 		agent_free(agent);
 
+	g_dbus_remove_watch(system_conn, listener_id);
+
+	if (any) {
+		g_free(any->path);
+		g_free(any);
+	}
+
 	if (system_conn)
 		dbus_connection_unref(system_conn);
 
@@ -587,55 +969,6 @@ int request_authorization(gint32 cid, int fd, const gchar *filename,
 	return 0;
 }
 
-gint add_record(gchar *record)
-{
-	DBusMessage *msg, *reply;
-	const gchar *adapter_path;
-	guint32 handle;
-
-	if (system_conn == NULL)
-		return 0;
-
-	msg = dbus_message_new_method_call("org.bluez", "/",
-			"org.bluez.Manager", "DefaultAdapter");
-
-	reply = dbus_connection_send_with_reply_and_block(system_conn, msg,
-			-1, NULL);
-	if (reply == NULL) {
-		dbus_message_unref(msg);
-		return 0;
-	}
-
-	dbus_message_unref(msg);
-
-	dbus_message_get_args(reply, NULL, DBUS_TYPE_OBJECT_PATH, &adapter_path,
-			DBUS_TYPE_INVALID);
-
-	dbus_message_unref(reply);
-
-	msg = dbus_message_new_method_call("org.bluez", adapter_path,
-			"org.bluez.Service", "AddRecord");
-
-	dbus_message_append_args(msg, DBUS_TYPE_STRING, &record,
-			DBUS_TYPE_INVALID);
-
-	reply = dbus_connection_send_with_reply_and_block(system_conn, msg,
-			-1, NULL);
-	if (reply == NULL) {
-		dbus_message_unref(msg);
-		return 0;
-	}
-
-	dbus_message_unref(msg);
-
-	dbus_message_get_args(reply, NULL, DBUS_TYPE_UINT32, &handle,
-			DBUS_TYPE_INVALID);
-
-	dbus_message_unref(reply);
-
-	return handle;
-}
-
 static void service_reply(DBusPendingCall *call, gpointer user_data)
 {
 	struct pending_request *pending = user_data;
@@ -644,32 +977,33 @@ static void service_reply(DBusPendingCall *call, gpointer user_data)
 
 	dbus_error_init(&derr);
 	if (dbus_set_error_from_message(&derr, reply)) {
-		error("Replied with an error: %s, %s",
+		error("RequestAuthorization error: %s, %s",
 				derr.name, derr.message);
 		dbus_error_free(&derr);
-		dbus_message_unref(reply);
-
 		close(pending->nsk);
-		g_free(pending);
-		return;
+		goto done;
 	}
 
+	debug("RequestAuthorization succeeded");
+
 	if (obex_session_start(pending->nsk, pending->server) < 0)
 		close(pending->nsk);
 
+done:
 	g_free(pending);
-
 	dbus_message_unref(reply);
 }
 
 static gboolean service_error(GIOChannel *io, GIOCondition cond,
 			gpointer user_data)
 {
-	gchar *adapter_path = user_data;
 	DBusMessage *msg;
 
+	if (any->path == NULL)
+		return FALSE;
+
 	msg = dbus_message_new_method_call("org.bluez",
-					adapter_path,
+					any->path,
 					"org.bluez.Service",
 					"CancelAuthorization");
 
@@ -677,25 +1011,80 @@ static gboolean service_error(GIOChannel *io, GIOCondition cond,
 
 	dbus_message_unref(msg);
 
-	g_free(adapter_path);
-
 	return FALSE;
 }
 
+static void find_adapter_reply(DBusPendingCall *call, gpointer user_data)
+{
+	struct pending_request *pending = user_data;
+	DBusMessage *reply = dbus_pending_call_steal_reply(call);
+	DBusMessage *msg;
+	DBusPendingCall *pcall;
+	GIOChannel *io;
+	guint watch;
+	const char *paddr = pending->address;
+	const char *path;
+	DBusError derr;
+
+	dbus_error_init(&derr);
+	if (dbus_set_error_from_message(&derr, reply)) {
+		error("Replied with an error: %s, %s",
+				derr.name, derr.message);
+		dbus_error_free(&derr);
+		goto done;
+	}
+
+	dbus_message_get_args(reply, NULL,
+			DBUS_TYPE_OBJECT_PATH, &path,
+			DBUS_TYPE_INVALID);
+
+	debug("FindAdapter -> %s", path);
+
+	msg = dbus_message_new_method_call("org.bluez", path,
+			"org.bluez.Service", "RequestAuthorization");
+
+	dbus_message_append_args(msg, DBUS_TYPE_STRING, &paddr,
+			DBUS_TYPE_UINT32, &pending->server->handle,
+			DBUS_TYPE_INVALID);
+
+	if (!dbus_connection_send_with_reply(system_conn,
+					msg, &pcall, TIMEOUT)) {
+		dbus_message_unref(msg);
+		goto done;
+	}
+
+	dbus_message_unref(msg);
+
+	debug("RequestAuthorization(%s, %x)", paddr, pending->server->handle);
+
+	if (!dbus_pending_call_set_notify(pcall, service_reply, pending, NULL)) {
+		close(pending->nsk);
+		g_free(pending);
+		goto done;
+	}
+
+	dbus_pending_call_unref(pcall);
+
+	/* Catches errors before authorization response comes */
+	io = g_io_channel_unix_new(pending->nsk);
+	watch = g_io_add_watch_full(io, G_PRIORITY_DEFAULT,
+			G_IO_HUP | G_IO_ERR | G_IO_NVAL,
+			service_error, NULL, NULL);
+	g_io_channel_unref(io);
+
+done:
+	dbus_message_unref(reply);
+}
+
 gint request_service_authorization(struct server *server, gint nsk)
 {
-	DBusMessage *msg, *reply;
-	DBusPendingCall *call;
-	GIOChannel *io;
 	struct sockaddr_rc src, dst;
 	socklen_t addrlen;
 	gchar address[18];
 	const gchar *paddr = address;
-	gchar *adapter_path, *path;
-	guint watch;
 	struct pending_request *pending;
 
-	if (system_conn == NULL)
+	if (system_conn == NULL || any->path == NULL)
 		return -1;
 
 	memset(&src, 0, sizeof(src));
@@ -707,75 +1096,20 @@ gint request_service_authorization(struct server *server, gint nsk)
 
 	ba2str(&src.rc_bdaddr, address);
 
-	msg = dbus_message_new_method_call("org.bluez", "/",
-			"org.bluez.Manager", "FindAdapter");
-
-	dbus_message_append_args(msg, DBUS_TYPE_STRING, &paddr,
-				DBUS_TYPE_INVALID);
-
-	reply = dbus_connection_send_with_reply_and_block(system_conn, msg,
-			-1, NULL);
-	if (reply == NULL) {
-		dbus_message_unref(msg);
+	if (getpeername(nsk, (struct sockaddr *) &dst, &addrlen) < 0)
 		return -1;
-	}
-
-	dbus_message_unref(msg);
-
-	dbus_message_get_args(reply, NULL, DBUS_TYPE_OBJECT_PATH, &path,
-			DBUS_TYPE_INVALID);
-
-	adapter_path = g_strdup(path);
-
-	dbus_message_unref(reply);
-
-	if (getpeername(nsk, (struct sockaddr *) &dst, &addrlen) < 0) {
-		g_free(adapter_path);
-		return -1;
-	}
-
-	ba2str(&dst.rc_bdaddr, address);
-
-	msg = dbus_message_new_method_call("org.bluez", adapter_path,
-			"org.bluez.Service", "RequestAuthorization");
-
-	dbus_message_append_args(msg, DBUS_TYPE_STRING, &paddr,
-			DBUS_TYPE_UINT32, &server->handle,
-			DBUS_TYPE_INVALID);
-
-	if (!dbus_connection_send_with_reply(system_conn,
-					msg, &call, TIMEOUT)) {
-		g_free(adapter_path);
-		dbus_message_unref(msg);
-		return -EPERM;
-	}
-
-	dbus_message_unref(msg);
 
 	pending = g_new0(struct pending_request, 1);
 	pending->server = server;
 	pending->nsk = nsk;
 
-	if (!dbus_pending_call_set_notify(call, service_reply, pending, NULL)) {
-		close(nsk);
-		g_free(adapter_path);
-		g_free(pending);
-		return -EPERM;
-	}
+	ba2str(&dst.rc_bdaddr, pending->address);
 
-	dbus_pending_call_unref(call);
-
-	/* Catches errors before authorization response comes */
-	io = g_io_channel_unix_new(nsk);
-	watch = g_io_add_watch_full(io, G_PRIORITY_DEFAULT,
-			G_IO_HUP | G_IO_ERR | G_IO_NVAL,
-			service_error, adapter_path, NULL);
-	g_io_channel_unref(io);
+	find_adapter(paddr, find_adapter_reply, pending);
 
 	return 0;
 }
 
-
 void register_session(guint32 id, struct obex_session *os)
 {
 	gchar *path = g_strdup_printf("/session%u", id);
