@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2007-2008  Nokia Corporation
  *  Copyright (C) 2007-2008  Instituto Nokia de Tecnologia (INdT)
- *  Copyright (C) 2007-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2007-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,24 +29,38 @@
 
 #include <glib.h>
 
-#define OBEX_OPUSH	0x00
-#define OBEX_FTP	0x01
+#include "phonebook.h"
+
+#define OBEX_OPP	(1 << 0)
+#define OBEX_FTP	(1 << 2)
+#define OBEX_BIP	(1 << 3)
+#define OBEX_PBAP	(1 << 4)
+#define OBEX_PCSUITE	(1 << 5)
 
 #define OBJECT_SIZE_UNKNOWN -1
 #define OBJECT_SIZE_DELETE -2
 
 struct obex_commands {
 	void (*get) (obex_t *obex, obex_object_t *obj);
-	gint (*chkput) (obex_t *obex, obex_object_t *obj);
 	void (*put) (obex_t *obex, obex_object_t *obj);
+	gint (*chkput) (obex_t *obex, obex_object_t *obj);
 	void (*setpath) (obex_t *obex, obex_object_t *obj);
 };
 
 struct server {
-	guint16		service;
+	guint16		services;
 	gboolean	auto_accept;
+	gchar		*name;
 	gchar		*folder;
 	gchar		*capability;
+	guint32		handle;
+	uint8_t		channel;
+	gchar		*devnode;
+	gboolean	secure;
+	GIOChannel	*io;
+	guint		watch;
+	guint16		tx_mtu;
+	guint16		rx_mtu;
 };
 
 struct obex_session {
@@ -67,19 +81,33 @@ struct obex_session {
 	struct obex_commands *cmds;
 	struct server *server;
 	gboolean	checked;
+	obex_t		*obex;
+	struct phonebook_context *pbctx;
+	gboolean	finished;
 };
 
 gint obex_session_start(gint fd, struct server *server);
 gint obex_session_stop();
 
-gint opp_chkput(obex_t *obex, obex_object_t *obj);
-void opp_put(obex_t *obex, obex_object_t *obj);
 void opp_get(obex_t *obex, obex_object_t *obj);
+void opp_put(obex_t *obex, obex_object_t *obj);
+gint opp_chkput(obex_t *obex, obex_object_t *obj);
 
 void ftp_get(obex_t *obex, obex_object_t *obj);
 void ftp_put(obex_t *obex, obex_object_t *obj);
-void ftp_setpath(obex_t *obex, obex_object_t *obj);
 gint ftp_chkput(obex_t *obex, obex_object_t *obj);
+void ftp_setpath(obex_t *obex, obex_object_t *obj);
+
+void pbap_get(obex_t *obex, obex_object_t *obj);
+void pbap_setpath(obex_t *obex, obex_object_t *obj);
+gboolean pbap_phonebook_context_create(struct obex_session *session);
+void pbap_phonebook_context_destroy(struct obex_session *session);
+struct obex_session *pbap_get_session(struct phonebook_context *context);
 
 gboolean os_prepare_get(struct obex_session *os, gchar *file, guint32 *size);
 gint os_prepare_put(struct obex_session *os);
+
+void server_free(struct server *server);
+
+int tty_init(gint service, const gchar *folder, const gchar *capability,
+		const gchar *devnode);
